﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.8
@EndOfDesignText@
'xCustomButton CustomView cross platform class.
'Version 1.0
'Author: Klaus CHRISTL (klaus)
'
'Events declarations
#Event: Click
#Event: LongClick
#Event: CheckdChange(Checked As Boolean)
#RaisesSynchronousEvents: Click
#RaisesSynchronousEvents: LongClick
#RaisesSynchronousEvents: CheckdChange

'Designer property declarations
#DesignerProperty: Key: ButtonType, DisplayName: ButtonType, FieldType: String, DefaultValue: STANDARD, List: STANDARD|TOGGLE, Description: Select the buttton type.
#DesignerProperty: Key: Text, DisplayName: Text, FieldType: String, DefaultValue: Text, Description: Text at the bottom of the button.
#DesignerProperty: Key: OnStateText, DisplayName: OnStateText, FieldType: String, DefaultValue: ON, Description: Text at the bottom of the button.
#DesignerProperty: Key: BackgroundColor, DisplayName: BackgroundColor, FieldType: Color, DefaultValue: 0x00FFFFFF, Description: Background color.
#DesignerProperty: Key: PressedColor, DisplayName: PressedColor, FieldType: Color, DefaultValue: 0xFFFFFF00, Description: Color when the button is pressed.
#DesignerProperty: Key: OnStateColor, DisplayName: OnStateColor, FieldType: Color, DefaultValue: 0xFFFF00FF, Description: Color for the ON state.
#DesignerProperty: Key: OnStateBackgroundColor, DisplayName: OnStateBackgroundColor, FieldType: Color, DefaultValue: 0x00FFFFFF, Description: Color of the background for the ON state.

'xCustomButton is a button based on a Panel with two Labels
'one with a Material Icon and the other with text.
'It has two events: Click and LongClick.
Sub Class_Globals
#If B4J
	Private fx As JFX
#End If
	Private xui As XUI

	Private mEventName As String
	Private mCallBack As Object
	Private xBase As B4XView
	
	Private mLeft, mTop, mWidth, mHeight As Int
	Private mText, mIcon As String
	
	Private xLabelFont, xIconFont As B4XFont
	
	Private mTextColor As Int
	Private mIconTextSize, mLabelTextSize As Double
	Private mTag As Object
	
	Private mLabel, mIconLabel As Label
	Private xLabel, xIconLabel As B4XView
	Private xParent As B4XView
	Private cvsBase As B4XCanvas
	Private rectBase As B4XRect
	Private mPressedColor, mBackgroundColor, mOnStateColor, mOnStateBackgroundColor As Int
	Private mONState = False As Boolean
	Private mButtonType As String
	Private mOnStateText As String
	
	Private tmrClick As Timer
	Private mTimeDown, mClickTime As Long	'used to distinguish Click and LongClick
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	
	mIcon = Chr(0xE859)
	mText = "Test"
	mTextColor = xui.Color_Black
	
	mClickTime = 400
	tmrClick.Initialize("tmrClick", mClickTime)
	
	mPressedColor = xui.Color_Yellow
	mButtonType = "STANDARD"
	mBackgroundColor = xui.Color_Transparent
	mOnStateColor = xui.Color_Blue
	
End Sub

Public Sub DesignerCreateView (Base As Object, Lbl As Label, Props As Map)
	Private mDummy As B4XView
	mDummy = Base
	mLeft = mDummy.Left
	mTop = mDummy.Top
	mWidth = mDummy.Width
	mHeight = mDummy.Height
	mIcon = Lbl.Text
	mText = Props.Get("Text")
	mPressedColor = xui.PaintOrColorToColor(Props.Get("PressedColor"))
	mBackgroundColor = xui.PaintOrColorToColor(Props.Get("BackgroundColor"))
	If mBackgroundColor = 16777215 Then
		mBackgroundColor = xui.Color_Transparent
	End If
	mButtonType = Props.Get("ButtonType")
	mOnStateText = Props.Get("OnStateText")
	mOnStateColor = xui.PaintOrColorToColor(Props.Get("OnStateColor"))
	mOnStateBackgroundColor = xui.PaintOrColorToColor(Props.Get("OnStateBackgroundColor"))
	
	xBase = xui.CreatePanel("xBase")
	xParent = mDummy.Parent
	xParent.AddView(xBase, mLeft, mTop, mWidth, mHeight)
	mTextColor = xui.PaintOrColorToColor(Lbl.TextColor)
	mTag = mDummy.Tag
	
'	mDummy.RemoveViewFromParent
#If B4A
	xIconFont = xui.CreateFont(Lbl.Typeface, Lbl.TextSize)
#Else
	xIconFont = Lbl.Font
#End If

	'InitClass
End Sub

Public Sub GetBase As Object
	Return xBase
End Sub

Public Sub AddToParent(Parent As Object, Left As Int, Top As Int, Width As Int, Height As Int, TextColor As Object, Icon As String, Text As String)
	mLeft = Left
	mTop = Top
	mWidth = Width
	mHeight = Height
	xParent = Parent

	xBase = xui.CreatePanel("xBase")
	
	xParent.AddView(xBase, mLeft, mTop, mWidth, mHeight)
	
	mIcon = Icon
	mText = Text
	mTextColor = xui.PaintOrColorToColor(TextColor)
	
	xIconFont = xui.CreateMaterialIcons(10)
	
	InitClass
End Sub

Private Sub InitClass
	'calculate the dimensions of the internal Labels
	Private lblLeft, lblWidth  As Int
	lblWidth = 2 * mHeight / 3		'icon Label width and height = 2/3 of button height
	lblLeft = (mWidth - lblWidth) / 2
	
	'initialize and add the icon Label
	mIconLabel.Initialize("")
	xIconLabel = mIconLabel
	
	mIconTextSize = mHeight / 2 / xui.Scale
	mLabelTextSize = lblWidth / 3 / xui.Scale
	xLabelFont = xui.CreateDefaultFont(10)
	
	xIconLabel.Font = xIconFont
	xIconLabel.SetTextAlignment("CENTER", "CENTER")
	xIconLabel.TextSize = mIconTextSize
	xIconLabel.TextColor = xui.PaintOrColorToColor(mTextColor)
	xBase.AddView(xIconLabel, lblLeft, 0, lblWidth, lblWidth)
	xIconLabel.Text = mIcon
	
	'initialize and add the text Label
	mLabel.Initialize("")
	xLabel = mLabel
	xLabel.SetTextAlignment("TOP", "CENTER")
	xLabel.Font = xLabelFont
	xLabel.TextSize = mLabelTextSize
	xLabel.TextColor = xui.PaintOrColorToColor(mTextColor)
	xBase.AddView(xLabel, 0, 2 * mHeight / 3, mWidth, mHeight / 3)
	xLabel.Text = mText
	
	cvsBase.Initialize(xBase)
	rectBase.Initialize(0, 0, xBase.Width, xBase.Height)
End Sub

Private Sub xBase_Touch(Action As Int, X As Float, Y As Float)
	Select Action
		Case xBase.TOUCH_ACTION_DOWN
			mTimeDown = DateTime.Now
			tmrClick.Enabled = True
			Select mButtonType
				Case "STANDARD"
					xBase.Color = mPressedColor
				Case "TOGGLE"
					If mONState = False Then
						xBase.Color = mOnStateBackgroundColor
						mONState = True
						xLabel.Text = mOnStateText
						xLabel.TextColor = mOnStateColor
'						xIconLabel.Text
						xIconLabel.TextColor = mOnStateColor
					Else
						If mBackgroundColor = xui.Color_Transparent Then
							cvsBase.ClearRect(rectBase)
							cvsBase.Invalidate
						Else
							xBase.Color = mBackgroundColor
						End If
						mONState = False
						xLabel.Text = mText
						xLabel.TextColor = mTextColor
						xIconLabel.TextColor = mTextColor
					End If
			End Select
		Case xBase.TOUCH_ACTION_UP
			Select mButtonType
				Case "STANDARD"
					If DateTime.Now - mTimeDown <= mClickTime Then
						If xui.SubExists(mCallBack, mEventName & "_Click", 0) Then
							CallSubDelayed(mCallBack, mEventName & "_Click")
							tmrClick.Enabled = False
						End If
					End If
					xBase.Color = mBackgroundColor
				Case "TOGGLE"
					If xui.SubExists(mCallBack, mEventName & "_CheckedChange", 1) Then
						CallSubDelayed2(mCallBack, mEventName & "_CheckedChange", mONState)
						tmrClick.Enabled = False
					End If
			End Select
	End Select
End Sub

Private Sub tmrClick_Tick
	If xui.SubExists(mCallBack, mEventName & "_LongClick", 0) Then
		CallSubDelayed(mCallBack, mEventName & "_LongClick")
		tmrClick.Enabled = False
	End If
End Sub

'set the icon Font
Public Sub setLabelFont(LabelFont As B4XFont)
	xLabelFont = LabelFont
	xLabel.Font = xLabelFont
	xLabel.TextSize = mLabelTextSize
End Sub

'set the icon Font
'should be "FontAwesome Icons or "Material Icons" Icons
Public Sub setIconFont(IconFont As B4XFont)
	xIconFont = IconFont
	xIconLabel.Font = xIconFont
	xIconLabel.TextSize = mIconTextSize
End Sub

'get or set the Tag property
Public Sub setTag(Tag As Object)
	mTag = Tag
	xBase.Tag = Tag
End Sub

Public Sub getTag As Object
	Return mTag
End Sub

'get or set the Icon property
Public Sub setIcon(Icon As String)
	xIconLabel.Text = Icon
End Sub

Public Sub getIcon As String
	Return xIconLabel.Text
End Sub

'get or set the Tag property
Public Sub setPressedColor(PressedColor As Int)
	mPressedColor = PressedColor
End Sub

Public Sub getPressedColor As Int
	Return mPressedColor
End Sub